package cn.haliaeetus.sendmessage.bean.delivery;

import cn.haliaeetus.sendmessage.constants.Constants;
import cn.haliaeetus.sendmessage.framework.log.Logger;
import org.apache.commons.lang3.ObjectUtils;

import static cn.haliaeetus.sendmessage.framework.util.RegexUtil.isValidPhoneNumberCN;
import static cn.haliaeetus.sendmessage.utils._.isEmpty;

public class DeliveryInfoUtil {
    private static final Logger LOG = Logger.getInstance(DeliveryInfoUtil.class);

    public static boolean isValidDeliveryInfo(DeliveryInfo deliveryInfo) {
        if (deliveryInfo == null) {
            throw new NullPointerException("DeliveryInfo is null.");
        }

        if (deliveryInfo.getDelivery() == null) {
            throw new NullPointerException("Delivery is null.");
        }

        if (deliveryInfo.getDeliveryStatus() == null) {
            throw new NullPointerException("DeliveryStatus is null.");
        }

        if (deliveryInfo.getStore() == null) {
            throw new NullPointerException("StoreInfo is null.");
        }

        if (deliveryInfo.getDelivery().getStoreId() == null || deliveryInfo.getDelivery().getStoreId() <= 0) {
            LOG.record(String.format("Delivery storeId is null, %s.", deliveryInfo));
            return false;
        }

        if (deliveryInfo.getDelivery().getStatus() == null) {
            LOG.record(String.format("Status of delivery is null, %s.", deliveryInfo));
            return false;
        }

        if (!isValidPhoneNumberCN(deliveryInfo.getDelivery().getUserPhone()) && 
        		!Constants.DEFAULT_MOBILE_KB.equals(deliveryInfo.getDelivery().getUserPhone())) {
            LOG.record(String.format("Invalid phone, %s.", deliveryInfo));
            return false;
        }

        return true;
    }

    /**
     * 是否自提派件
     */
    public static boolean isDeliveryThatTakeByCustomer(DeliveryInfo deliveryInfo) {
        if (!isValidDeliveryInfo(deliveryInfo)) {
            return false;
        }
        return ObjectUtils.equals(deliveryInfo.getDelivery().getOdeliveryType(), Constants.ORDER_SEND_TYPE_ZT);
    }

    /**
     * 是否外出派送派件
     */
    public static boolean isDeliveryThatOutDispatch(DeliveryInfo deliveryInfo) {
        if (!isValidDeliveryInfo(deliveryInfo)) {
            return false;
        }
        return ObjectUtils.equals(deliveryInfo.getDelivery().getOdeliveryType(), Constants.ORDER_SEND_TYPE_PS);
    }

    /**
     * 是否P2P派件
     */
    public static boolean isDeliveryThatTakeByP2P(DeliveryInfo deliveryInfo) {
        if (!isValidDeliveryInfo(deliveryInfo)) {
            return false;
        }
        return ObjectUtils.equals(deliveryInfo.getDelivery().getOdeliveryType(), Constants.ORDER_SEND_TYPE_P2P);
    }

    /**
     * 是否派件入库
     */
    public static boolean isDeliveryInStore(DeliveryInfo deliveryInfo) {
        if (!isValidDeliveryInfo(deliveryInfo)) {
            return false;
        }

        if (ObjectUtils.equals(deliveryInfo.getDelivery().getStatus(), Constants.ORDER_STATUS_STORED)) {
            if (deliveryInfo.getDeliveryStatus().getPreStatus() == null ||
                deliveryInfo.getDeliveryStatus().getPreStatus() == 0 ||
                ObjectUtils.equals(deliveryInfo.getDeliveryStatus().getPreStatus(), Constants.ORDER_STATUS_AFLOAT) ||
                ObjectUtils.equals(deliveryInfo.getDeliveryStatus().getPreStatus(), Constants.ORDER_STATUS_WAIT_IN_STORE) ||
                ObjectUtils.equals(deliveryInfo.getDeliveryStatus().getPreStatus(), Constants.ORDER_STATUS_STORED) ||
                ObjectUtils.equals(deliveryInfo.getDeliveryStatus().getPreStatus(), Constants.ORDER_STATUS_LEAVE_STORE) ||
                ObjectUtils.equals(deliveryInfo.getDeliveryStatus().getPreStatus(), Constants.ORDER_STATUS_REJECT_LEAVE_STORE)) {
                return true;
            }
        }

        return false;
    }

    /**
     * 是否派件手机号被完善或发生变化
     */
    public static boolean isDeliveryMobilephoneChange(DeliveryInfo deliveryInfo) {
        if (!isValidDeliveryInfo(deliveryInfo)) {
            return false;
        }

        if (deliveryInfo.getOldDelivery() == null || isEmpty(deliveryInfo.getOldDelivery().getUserPhone())) {
            return true;
        }

        return ObjectUtils.notEqual(deliveryInfo.getDelivery().getUserPhone(), deliveryInfo.getOldDelivery().getUserPhone());
    }

    /**
     * 是否派件带有合法手机号入库
     */
    public static boolean isDeliveryInStoreWithMobilephone(DeliveryInfo deliveryInfo) {
        if (!isValidDeliveryInfo(deliveryInfo)) {
            return false;
        }

        if (!isDeliveryInStore(deliveryInfo)) {
            return false;
        }

        if (isDeliveryMobilephoneChange(deliveryInfo)) {
            LOG.record(String.format("Delivery instore with mobilephone change, %s.", deliveryInfo));
            return true;
        }

        if (isDeliveryStatusChange(deliveryInfo)) {
            LOG.record(String.format("Delivery that contains mobilephone instore with status change, %s.", deliveryInfo));
            return true;
        }

        return false;
    }

    /**
     * 是否派件状态发生变化
     */
    public static boolean isDeliveryStatusChange(DeliveryInfo deliveryInfo) {
        if (!isValidDeliveryInfo(deliveryInfo)) {
            return false;
        }

        return ObjectUtils.notEqual(deliveryInfo.getDelivery().getStatus(), deliveryInfo.getOldDelivery().getStatus());
    }

    /**
     * 是否派件货架转移
     */
    public static boolean isDeliveryShelfChange(DeliveryInfo deliveryInfo) {
        if (!isValidDeliveryInfo(deliveryInfo) || deliveryInfo.getOldDelivery() == null) {
            return false;
        }

        if (ObjectUtils.equals(deliveryInfo.getDelivery().getStatus(), Constants.ORDER_STATUS_STORED) &&
            ObjectUtils.equals(deliveryInfo.getDeliveryStatus().getPreStatus(), Constants.ORDER_STATUS_STORED)) {

            Integer shelfId = deliveryInfo.getDelivery().getShelfId();
            Integer oldShelfId = deliveryInfo.getOldDelivery().getShelfId();

            if (shelfId != null && oldShelfId != null && !shelfId.equals(oldShelfId)) {
                return true;
            }
        }

        return false;
    }

    /**
     * 是否派件外出配送
     */
    public static boolean isDeliveryDispatching(DeliveryInfo deliveryInfo) {
        if (!isValidDeliveryInfo(deliveryInfo)) {
            return false;
        }

        if (ObjectUtils.notEqual(deliveryInfo.getDelivery().getStatus(), Constants.ORDER_STATUS_DISPATCHING)) {
            return false;
        }

        if (ObjectUtils.equals(deliveryInfo.getDeliveryStatus().getPreStatus(), Constants.ORDER_STATUS_AFLOAT) ||
            ObjectUtils.equals(deliveryInfo.getDeliveryStatus().getPreStatus(), Constants.ORDER_STATUS_STORED) ||
            ObjectUtils.equals(deliveryInfo.getDeliveryStatus().getPreStatus(), Constants.ORDER_STATUS_DISPATCHING) ||
            ObjectUtils.equals(deliveryInfo.getDeliveryStatus().getPreStatus(), Constants.ORDER_STATUS_REJECT_PICK_UP)) {
            return true;
        }

        return false;
    }
}