package cn.haliaeetus.sendmessage.message.consumer;

import cn.haliaeetus.sendmessage.bean.basedata.User;
import cn.haliaeetus.sendmessage.bean.delivery.Delivery;
import cn.haliaeetus.sendmessage.bean.dto.DeliveryDto;
import cn.haliaeetus.sendmessage.bean.sms.SMSSendQueue;
import cn.haliaeetus.sendmessage.dao.SmsRecordDao;
import cn.haliaeetus.sendmessage.framework.toolkit.CacheTool;
import cn.haliaeetus.sendmessage.service.BasicInfoService;
import cn.haliaeetus.sendmessage.service.MessageSendService;
import cn.haliaeetus.sendmessage.service.PlatformConfigService;
import cn.haliaeetus.sendmessage.service.UserService;
import cn.haliaeetus.sendmessage.service.sms.SmsRecordService;
import cn.haliaeetus.sendmessage.utils.ValidationUtils;
import cn.haliaeetus.sendmessage.utils.config.RabbitConfig;
import cn.haliaeetus.sendmessage.validate.groups.SendSms;
import com.alibaba.fastjson.JSON;
import com.google.gson.Gson;
import com.rabbitmq.client.Channel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.rabbit.annotation.*;
import org.springframework.amqp.support.AmqpHeaders;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.messaging.handler.annotation.Headers;
import org.springframework.messaging.handler.annotation.Payload;
import org.springframework.stereotype.Component;

import java.util.Map;

/**
 * @ClassName SMSSendQueueListener
 * @Description 短信消息消费者
 * @Author kael
 * @Date 2019/6/25 0025 下午 14:20
 * @Version 1.0
 **/
@Component
public class SMSSendQueueListener {
    Logger log = LoggerFactory.getLogger(SMSSendQueueListener.class);
    @Autowired
    private MessageSendService messageSendService;

    @Autowired
    private UserService userService;

    @Autowired
    private BasicInfoService basicService;

    @Autowired
    private SmsRecordService smsRecordService;

    @Value("${spring.rabbitmq.smsQueueName}")
    private String SMS_QUEUE;

    @Autowired
    private RabbitConfig rabbitConfig;

    @Autowired
    private SmsRecordDao smsRecordDao;

    @Autowired
    private PlatformConfigService platformConfigService;
    /**
     * 等待回执
     */
    private static final int WAITINGRECEIPT = 1;
    /**
     * 重发次数
     */
    private static final int NUMBERRESENDS = 8;

    /**
     * 配置监听的哪一个队列，同时在没有queue和exchange的情况下会去创建并建立绑定关系
     *
     * @param jsonObject
     * @param headers
     * @param channel
     */
    @RabbitListener(bindings = @QueueBinding(
            value = @Queue(value = "${spring.rabbitmq.smsQueueName}",durable = "true"),
            exchange = @Exchange(name="${spring.rabbitmq.smsExchangeName}",durable = "true",type = "direct"),
            key = "${spring.rabbitmq.smsRoutingKey}"
    ))
    @RabbitHandler
    public void receive(@Payload String jsonObject, @Headers Map<String, Object> headers, Channel channel) {
        SMSSendQueue smsSendQueue = null;
        try {
            // 采用手动应答模式, 手动确认应答更为安全稳定
            channel.basicAck((Long) headers.get(AmqpHeaders.DELIVERY_TAG), true);
            //添加mq日志
            smsRecordService.installApiLogMq(jsonObject, "smsQueue");
            log.info("短信mq传入参数：" + jsonObject);
            smsSendQueue = this.getSmsSendQueue(jsonObject, smsSendQueue);
            smsMsgHandle(smsSendQueue);
        } catch (Exception e) {
            this.msgErrorHandle(smsSendQueue, e.getMessage());
            e.printStackTrace();
        }
    }

    // 短信发送异常
    private void msgErrorHandle(SMSSendQueue smsSendQueue, String exception) {
        ValidationUtils.ValidationResult validationResult = ValidationUtils.validate(smsSendQueue, SendSms.class);
        if (!validationResult.hasErrors) {
            DeliveryDto dto = smsSendQueue.getDeliveryDtos();
            String dxKey = getDxKey(dto.getDelivery());
            CacheTool cacheTool = CacheTool.initSmsFlagTool();
            // 删除dxKey
            cacheTool.del(dxKey);
            // 修改delivery表中的数据为发送失败
            Delivery param = new Delivery();
            param.setId(dto.getDelivery().getId());
            // 发送失败
            param.setSmsStatus(2);
            if (null != exception && exception.length() > 100) {
                exception = exception.substring(0, 100);
            }
            param.setResultInfo(exception);
            smsRecordDao.updateSmsStatusById(param);
        }
    }

    /**
     * 双重转换防止意外报错
     *
     * @param jsonObject
     * @param smsSendQueue
     * @return
     */
    private SMSSendQueue getSmsSendQueue(@Payload String jsonObject, SMSSendQueue smsSendQueue) {
        try {
            smsSendQueue = new Gson().fromJson(jsonObject, SMSSendQueue.class);
        } catch (Exception e) {
            smsSendQueue = JSON.parseObject(jsonObject, SMSSendQueue.class);
        }
        return smsSendQueue;
    }

    public boolean smsMsgHandle(SMSSendQueue smsSendQueue) throws Exception {
        ValidationUtils.ValidationResult validationResult = ValidationUtils.validate(smsSendQueue, SendSms.class);
        if (validationResult.hasErrors) {
            return true;
        }

        DeliveryDto dto = smsSendQueue.getDeliveryDtos();
        String dxKey = getDxKey(dto.getDelivery());
        CacheTool cacheTool = CacheTool.initSmsFlagTool();
        String smsFlag = cacheTool.getStringKey(dxKey);
        if (dxKey == null || !"0".equals(smsFlag)) {
            log.warn("kale --------->：redis中key为：" + dxKey + "的值不为0或者不存在");
            return true;
        }
        Integer SMSFlagData = Integer.parseInt(basicService.getConfigurationByCode("SMSFlag").getOValue1());
        cacheTool.setStringandDatakey(dxKey, "1", SMSFlagData * 60 * 60);

        User temUser = userService.getUserByuserId(smsSendQueue.getUser().getUserId());
        log.info("准备发送短信===============");
        messageSendService.syncResendInStoreMsg(dto.getDelivery(), dto.getSmsStatus(), dto.getBussCode(), temUser);
        return true;
    }

    private String getDxKey(Delivery delivery) {
        return String.format("%d_%s_%s_%d", delivery.getExpId(), delivery.getExpcode(), delivery.getUserPhone(), delivery.getStoreId());
    }
}
