package cn.haliaeetus.sendmessage.utils.config;

import lombok.Data;
import org.springframework.amqp.core.Binding;
import org.springframework.amqp.core.BindingBuilder;
import org.springframework.amqp.core.DirectExchange;
import org.springframework.amqp.core.Queue;
import org.springframework.amqp.rabbit.connection.ConnectionFactory;
import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;

/**
 * @ClassName RabbitConfig
 * @Description rabbitmq设置
 * @Author kael
 * @Date 2019/8/13 0013 下午 14:45
 * @Version 1.0
 **/
@Data
@Component
@ConfigurationProperties(prefix = "spring.rabbitmq")
@Configuration
public class RabbitConfig {

    private String smsQuery;
    private String smsQueryExchange;
    private String smsQueryKey;

    /******死信队列******/
    private String smsDeadQuery;
    private String smsDeadQueryExchange;
    private String smsDeadQueryKey;

    /**
     * 短信
     */
    private String apiLogQueueName;
    private String apiLogExchangeName;
    private String apiLogRoutingKey;

    /**
     * 极光推送
     */
    private String jPushQueue;
    private String jPushExchange;
    private String jPushRoutingKey;

    /**
     * PhoneLib
     */
    private String phoneLibQueue;
    private String phoneLibExchange;
    private String phoneLibRoutingKey;

    /**
     * PhoneLib
     */
    private String smsQueueName;
    private String smsExchangeName;
    private String smsRoutingKey;


    /**
     * 死信队列交换机标识符
     */
    public static final String DEAD_LETTER_QUEUE_KEY = "x-dead-letter-exchange";
    /**
     * 死信队列交换机绑定键标识符
     */
    public static final String DEAD_LETTER_ROUTING_KEY = "x-dead-letter-routing-key";

    @Bean
    public RabbitAdmin rabbitAdmin(ConnectionFactory connectionFactory) {
        RabbitAdmin rabbitAdmin = new RabbitAdmin(connectionFactory);
        //设置忽略声明异常
        rabbitAdmin.setIgnoreDeclarationExceptions(true);
        return rabbitAdmin;
    }

    /**
     * 创建配置死信队列
     *
     * @return
     */
    @Bean
    public Queue deadQueue() {
        Queue queue = new Queue(smsDeadQuery, true);
        return queue;
    }
    /*
     * 创建死信交换机
     */
    @Bean
    DirectExchange deadExchange() {
        return new DirectExchange(smsDeadQueryExchange,true,false);
    }
    /*
     * 死信队列与死信交换机绑定
     */
    @Bean
    public Binding bindingDeadExchange(Queue deadQueue, DirectExchange deadExchange) {
        return BindingBuilder.bind(deadQueue).to(deadExchange).with(smsDeadQueryKey);
    }

    // 1.定义短信队列
    @Bean
    public Queue fanOutSmsQueue() {
        // 将普通队列绑定到死信队列交换机上
        Map<String, Object> args = new HashMap<>(2);
        args.put(DEAD_LETTER_QUEUE_KEY, smsDeadQueryExchange);
        args.put(DEAD_LETTER_ROUTING_KEY, smsDeadQueryKey);
//        args.put("x-message-ttl",50000);
        Queue queue = new Queue(smsQuery, true, false, false, args);
        return queue;
    }

    // 1.定义短信队列
    @Bean
    public Queue fanSmsQueue() {
        return new Queue(smsQuery,true);
    }

    // 2.定义交换机
    @Bean
    DirectExchange smsExchange() {
        return new DirectExchange(smsQueryExchange,true,false);
    }


    // 3.队列与交换机绑定短信队列
    @Bean
    public Binding bindingSMSExchange(Queue fanSmsQueue, DirectExchange smsExchange) {
        return BindingBuilder.bind(fanSmsQueue).to(smsExchange).with(smsQueryKey);
    }

    // 1.定义日志队列
    @Bean
    public Queue apiQueue() {
        return new Queue(apiLogQueueName,true);
    }

    // 2.定义日志交换机
    @Bean
    DirectExchange apiExchange() {
        return new DirectExchange(apiLogExchangeName,true,false);
    }

    // 3.队列与交换机绑定日志队列
    @Bean
    public Binding bindingapiExchange(Queue apiQueue, DirectExchange apiExchange) {
        return BindingBuilder.bind(apiQueue).to(apiExchange).with(apiLogRoutingKey);
    }

    // 1.定义日志队列
    @Bean
    public Queue jPushQueue() {
        return new Queue(jPushQueue,true);
    }

    // 2.定义日志交换机
    @Bean
    DirectExchange jPushExchange() {
        return new DirectExchange(jPushExchange,true,false);
    }

    // 3.队列与交换机绑定日志队列
    @Bean
    public Binding bindingjPushExchange(Queue jPushQueue, DirectExchange jPushExchange) {
        return BindingBuilder.bind(jPushQueue).to(jPushExchange).with(jPushRoutingKey);
    }

    // 1.PhoneLib队列
    @Bean
    public Queue phoneLibQueue() {
        return new Queue(phoneLibQueue,true);
    }

    // 2.PhoneLib交换机
    @Bean
    DirectExchange phoneLibExchange() {
        return new DirectExchange(phoneLibExchange,true,false);
    }

    // 3.PhoneLib队列与交换机绑定
    @Bean
    public Binding bindingPhoneLibExchange(Queue phoneLibQueue, DirectExchange phoneLibExchange) {
        return BindingBuilder.bind(phoneLibQueue).to(phoneLibExchange).with(phoneLibRoutingKey);
    }

}
